import os
import shutil
import tempfile
from unittest import mock, TestCase

from gtsrvd import nginx


meta_ip_response = mock.MagicMock(text="192.168.1.100")


class TestNginx(TestCase):
    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()
        self.subdomain = "test1"

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    @mock.patch("gtsrvd.nginx.call")
    def test_create_proxy(self, call):
        with mock.patch("gtsrvd.nginx.NGINX_CONF_ROOT", self.temp_dir):
            conf_path = nginx.proxy_conf_path(self.subdomain)
            nginx.create_proxy("blastedstudios.com", self.subdomain, 8081)
            self.assertTrue(os.path.exists(conf_path))

    @mock.patch("gtsrvd.nginx.call")
    @mock.patch("gtsrvd.nginx.os")
    def test_delete_proxy(self, os, call):
        with mock.patch("gtsrvd.nginx.NGINX_CONF_ROOT", self.temp_dir):
            nginx.delete_proxy("blastedstudios.com", self.subdomain, 8081)
            self.assertTrue(call.called)
            self.assertTrue(os.remove.called)



#!/usr/bin/env python

import click

from gtsrvd import app


@click.command()
@click.option("--domain", default="example.com", help="Target domain")
@click.option("--subdomain", default="test1", help="Target subdomain")
@click.option("--port", default=80, help="Target port to proxy to")
def create(domain, subdomain, port):
    """ Create proxy to localhost on port. Will add dns record to domain. """
    app.create(domain, subdomain, int(port))


@click.command()
@click.option("--domain", default="example.com", help="Target domain")
@click.option("--subdomain", default="test1", help="Target subdomain")
@click.option("--port", default=80, help="Target port to proxy to")
def delete(domain, subdomain, port):
    """ Delete proxy to localhost on port. Will remove dns record from domain. """
    app.delete(domain, subdomain, int(port))



import boto3
import requests


client = boto3.client("route53")
METADATA_URL = "http://169.254.169.254/latest/meta-data/"
METADATA_IP_URL = METADATA_URL + "public-ipv4/"


def change_record(domain, subdomain, target_ip, action, ttl=900):
    """ Change the record for subdomain """
    zone_id = get_hosted_zone_id(domain)
    name = subdomain + "." + domain
    client.change_resource_record_sets(
        HostedZoneId=zone_id,
        ChangeBatch={
            "Comment": "%s subdomain %s from zone %s" % (action, subdomain, zone_id),
            "Changes": [
                {
                    "Action": action,
                    "ResourceRecordSet": {
                        "Name": name,
                        "Type": "A",
                        "ResourceRecords": [{"Value": target_ip}],
                        "TTL": ttl,
                    },
                }
            ],
        },
    )


def get_hosted_zone_id(domain):
    name = ".".join(reversed(domain.split(".")))
    result = client.list_hosted_zones_by_name(DNSName=name, MaxItems="1")
    return result["HostedZones"][0]["Id"]


def create_record(domain, subdomain):
    """ Create a new A record to this machine for subdomain
    :param string domain: name for domain, e.g. blastedstudios.com.
    :param string subdomain: subdomain name
    """
    change_record(domain, subdomain, get_ip(), "CREATE")


def delete_record(domain, subdomain):
    """ Delete the record for subdomain
    :param string domain: name for domain, e.g. blastedstudios.com.
    :param string subdomain: subdomain name
    """
    change_record(domain, subdomain, get_ip(), "DELETE")


def get_ip():
    """ Return this instance's ip"""
    return requests.get(METADATA_IP_URL).text



from unittest import mock, TestCase

from gtsrvd import aws


meta_ip_response = mock.MagicMock(text="192.168.1.100")


class TestAWS(TestCase):
    @mock.patch("gtsrvd.aws.client")
    @mock.patch("gtsrvd.aws.requests.get", return_value=meta_ip_response)
    def test_create(self, requests_get, client):
        aws.create_record("blastedstudios.com", "test1")
        self.assertTrue(client.change_resource_record_sets.called)

    @mock.patch("gtsrvd.aws.client")
    @mock.patch("gtsrvd.aws.requests.get", return_value=meta_ip_response)
    def test_delete(self, requests_get, client):
        aws.delete_record("blastedstudios.com", "test1")
        self.assertTrue(client.change_resource_record_sets.called)



import os
from subprocess import call

NGINX_CONF_ROOT = "/etc/nginx/conf.d/"
NGINX_CONF_TEMPLATE = """
server {
    listen       80;
    server_name  %s;

    location / {
        proxy_pass http://127.0.0.1:%d;
    }
}
"""


def create_proxy(domain, subdomain, port):
    """ Create nginx proxy_pass for the given domain on given port
    :param string domain: domain of item, e.g. blastedstudios.com
    :param string subdomain: desired subdomain of item, e.g. test1
    :param string port: port of server for proxy (generated by ssh remote forward)
    """
    server_name = subdomain + "." + domain
    nginx_conf_text = NGINX_CONF_TEMPLATE % (server_name, port)
    nginx_conf_path = proxy_conf_path(subdomain)
    # assume we have permissions - percolate exception otherwise
    with open(nginx_conf_path, "w") as conf:
        conf.write(nginx_conf_text)
    # after writing new conf, we need to reload nginx
    call(["nginx", "-s", "reload"])


def delete_proxy(domain, subdomain, port):
    """ Remove previously created proxy for domain on port """
    nginx_conf_path = proxy_conf_path(subdomain)
    os.remove(nginx_conf_path)
    call(["nginx", "-s", "reload"])


def proxy_conf_path(subdomain):
    """ Get path for given subdomain """
    return os.path.join(NGINX_CONF_ROOT, subdomain + ".conf")



from gtsrvd import aws, nginx

cloud = aws  # "eventually" will be cloud agnostic (?)


def create(domain, subdomain, port):
    cloud.create_record(domain, subdomain)
    nginx.create_proxy(domain, subdomain, port)


def delete(domain, subdomain, port):
    cloud.delete_record(domain, subdomain)
    nginx.delete_proxy(domain, subdomain, port)



from setuptools import setup, find_packages


setup(
    name="gtsrvd",
    version="0.1.1",
    description=("gtsrvd proxies localhost to public internet"),
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ],
    keywords="aws ngrok serveo",
    author="Jon Robison",
    author_email="narfman0@gmail.com",
    url="https://github.com/narfman0/gtsrvd",
    license="LICENSE",
    packages=find_packages(),
    include_package_data=True,
    install_requires=["boto3", "requests", "click"],
    zip_safe=True,
    test_suite="tests",
    entry_points={
        "console_scripts": [
            "gtsrvd-create=gtsrvd.cli:create",
            "gtsrvd-delete=gtsrvd.cli:delete",
        ]
    },
)
